"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConfiguration = void 0;
const locate_file_1 = require("../configuration/locate_file");
const configuration_1 = require("../configuration");
const validate_configuration_1 = require("../configuration/validate_configuration");
const convert_configuration_1 = require("./convert_configuration");
const environment_1 = require("./environment");
const console_logger_1 = require("./console_logger");
/**
 * Load user-authored configuration to be used in a test run.
 *
 * @public
 * @param options - Coordinates required to find configuration.
 * @param environment - Project environment.
 */
async function loadConfiguration(options = {}, environment = {}) {
    var _a;
    const { cwd, stderr, env, debug } = (0, environment_1.mergeEnvironment)(environment);
    const logger = new console_logger_1.ConsoleLogger(stderr, debug);
    const configFile = (_a = options.file) !== null && _a !== void 0 ? _a : (0, locate_file_1.locateFile)(cwd);
    if (configFile) {
        logger.debug(`Configuration will be loaded from "${configFile}"`);
    }
    else {
        logger.debug('No configuration file found');
    }
    const profileConfiguration = configFile
        ? await (0, configuration_1.fromFile)(logger, cwd, configFile, options.profiles)
        : {};
    const original = (0, configuration_1.mergeConfigurations)(configuration_1.DEFAULT_CONFIGURATION, profileConfiguration, options.provided);
    logger.debug('Resolved configuration:', original);
    (0, validate_configuration_1.validateConfiguration)(original);
    const runnable = await (0, convert_configuration_1.convertConfiguration)(original, env);
    return {
        useConfiguration: original,
        runConfiguration: runnable,
    };
}
exports.loadConfiguration = loadConfiguration;
//# sourceMappingURL=load_configuration.js.map