"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.publishPlugin = void 0;
const http_stream_1 = __importDefault(require("./http_stream"));
const stream_1 = require("stream");
const supports_color_1 = require("supports-color");
const has_ansi_1 = __importDefault(require("has-ansi"));
const strip_ansi_1 = __importDefault(require("strip-ansi"));
const DEFAULT_CUCUMBER_PUBLISH_URL = 'https://messages.cucumber.io/api/reports';
const publishPlugin = async ({ on, logger, configuration, environment, }) => {
    if (!configuration.formats.publish) {
        return undefined;
    }
    const { url = DEFAULT_CUCUMBER_PUBLISH_URL, token } = configuration.formats.publish;
    const headers = {};
    if (token !== undefined) {
        headers.Authorization = `Bearer ${token}`;
    }
    const stream = new http_stream_1.default(url, 'GET', headers);
    const readerStream = new stream_1.Writable({
        objectMode: true,
        write: function (responseBody, encoding, writeCallback) {
            environment.stderr.write(sanitisePublishOutput(responseBody, environment.stderr) + '\n');
            writeCallback();
        },
    });
    stream.pipe(readerStream);
    stream.on('error', (error) => logger.error(error.message));
    on('message', (value) => stream.write(JSON.stringify(value) + '\n'));
    return () => stream.end();
};
exports.publishPlugin = publishPlugin;
/*
This is because the Cucumber Reports service returns a pre-formatted console message
including ANSI escapes, so if our stderr stream doesn't support those we need to
strip them back out. Ideally we should get structured data from the service and
compose the console message on this end.
 */
function sanitisePublishOutput(raw, stderr) {
    if (!(0, supports_color_1.supportsColor)(stderr) && (0, has_ansi_1.default)(raw)) {
        return (0, strip_ansi_1.default)(raw);
    }
    return raw;
}
//# sourceMappingURL=publish_plugin.js.map