/// <reference types="node" />
/// <reference types="node" />
import { Readable } from 'stream';
import * as messages from '@cucumber/messages';
export interface IAttachmentMedia {
    encoding: messages.AttachmentContentEncoding;
    contentType: string;
}
export interface IAttachment {
    data: string;
    media: IAttachmentMedia;
}
export type IAttachFunction = (attachment: IAttachment) => void;
export type ICreateStringAttachment = (data: string, mediaType?: string) => void;
export type ICreateBufferAttachment = (data: Buffer, mediaType: string) => void;
export type ICreateStreamAttachment = (data: Readable, mediaType: string) => Promise<void>;
export type ICreateStreamAttachmentWithCallback = (data: Readable, mediaType: string, callback: () => void) => void;
export type ICreateAttachment = ICreateStringAttachment & ICreateBufferAttachment & ICreateStreamAttachment & ICreateStreamAttachmentWithCallback;
export type ICreateLog = (text: string) => void;
export default class AttachmentManager {
    private readonly onAttachment;
    constructor(onAttachment: IAttachFunction);
    log(text: string): void | Promise<void>;
    create(data: Buffer | Readable | string, mediaType?: string, callback?: () => void): void | Promise<void>;
    createBufferAttachment(data: Buffer, mediaType: string): void;
    createStreamAttachment(data: Readable, mediaType: string, callback: () => void): void | Promise<void>;
    createStringAttachment(data: string, media: IAttachmentMedia): void;
}
